@@ -1044,6 +1044,7 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
     XfceNotifyWindow *window;
     GdkPixbuf *pix = NULL;
     GVariant *image_data = NULL;
+    const gchar *image_path = NULL;
     const gchar *desktop_id = NULL;
     gint value_hint = 0;
     gboolean value_hint_set = FALSE;
@@ -1080,6 +1081,11 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
         {
             image_data = value;
         }
+        else if ((g_strcmp0 (key, "image-path") == 0) ||
+                 (g_strcmp0 (key, "image_path") == 0))
+        {
+            image_path = g_variant_get_string (value, NULL);
+        }
         else if ((g_strcmp0 (key, "desktop_entry") == 0) ||
                  (g_strcmp0 (key, "desktop-entry") == 0))
         {
@@ -1107,10 +1113,9 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
        && (window = g_tree_lookup(xndaemon->active_notifications,
                                   GUINT_TO_POINTER(replaces_id))))
     {
-        xfce_notify_window_set_icon_name(window, app_icon);
         xfce_notify_window_set_summary(window, summary);
         xfce_notify_window_set_body(window, body);
-        xfce_notify_window_set_actions(window, actions);
+        xfce_notify_window_set_actions(window, actions, xndaemon->css_provider);
         xfce_notify_window_set_expire_timeout(window, expire_timeout);
         xfce_notify_window_set_opacity(window, xndaemon->initial_opacity);
 
@@ -1119,7 +1124,8 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
         window = XFCE_NOTIFY_WINDOW(xfce_notify_window_new_with_actions(summary, body,
                                                                         app_icon,
                                                                         expire_timeout,
-                                                                        actions));
+                                                                        actions,
+                                                                        xndaemon->css_provider));
         xfce_notify_window_set_opacity(window, xndaemon->initial_opacity);
 
         OUT_id = xfce_notify_daemon_generate_id(xndaemon);
@@ -1146,43 +1152,48 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
         g_idle_add(notify_show_window, window);
     }
 
-    if(!app_icon || !*app_icon) {
-        if(image_data) {
-            pix = notify_pixbuf_from_image_data(image_data);
-            if(pix) {
-                xfce_notify_window_set_icon_pixbuf(window, pix);
-                g_object_unref(G_OBJECT(pix));
-            }
-            g_variant_unref(image_data);
-        } else {
-            if(desktop_id) {
-                gchar *resource = g_strdup_printf("applications%c%s.desktop",
-                                                  G_DIR_SEPARATOR,
-                                                  desktop_id);
-                XfceRc *rcfile = xfce_rc_config_open(XFCE_RESOURCE_DATA,
-                                                     resource, TRUE);
-                if(rcfile) {
-                    if(xfce_rc_has_group(rcfile, "Desktop Entry")) {
-                        const gchar *icon_file;
-                        xfce_rc_set_group(rcfile, "Desktop Entry");
-                        icon_file = xfce_rc_read_entry(rcfile, "Icon", NULL);
-                        if(icon_file) {
-                            pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                                           icon_file,
-                                                           32,
-                                                           GTK_ICON_LOOKUP_FORCE_SIZE,
-                                                           NULL);
+    if (image_data) {
+        pix = notify_pixbuf_from_image_data(image_data);
+        if(pix) {
+            xfce_notify_window_set_icon_pixbuf(window, pix);
+            g_object_unref(G_OBJECT(pix));
+        }
+        g_variant_unref(image_data);
+    }
+    else if (image_path) {
+        xfce_notify_window_set_icon_name (window, image_path);
+    }
+    else if (app_icon) {
+        xfce_notify_window_set_icon_name(window, app_icon);
+    }
+    else {
+        if(desktop_id) {
+            gchar *resource = g_strdup_printf("applications%c%s.desktop",
+                                              G_DIR_SEPARATOR,
+                                              desktop_id);
+            XfceRc *rcfile = xfce_rc_config_open(XFCE_RESOURCE_DATA,
+                                                 resource, TRUE);
+            if(rcfile) {
+                if(xfce_rc_has_group(rcfile, "Desktop Entry")) {
+                    const gchar *icon_file;
+                    xfce_rc_set_group(rcfile, "Desktop Entry");
+                    icon_file = xfce_rc_read_entry(rcfile, "Icon", NULL);
+                    if(icon_file) {
+                        pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
+                                                       icon_file,
+                                                       GTK_ICON_SIZE_DIALOG,
+                                                       GTK_ICON_LOOKUP_FORCE_SIZE,
+                                                       NULL);
 
-                            if(pix) {
-                                xfce_notify_window_set_icon_pixbuf(window, pix);
-                                g_object_unref(G_OBJECT(pix));
-                            }
+                        if(pix) {
+                            xfce_notify_window_set_icon_pixbuf(window, pix);
+                            g_object_unref(G_OBJECT(pix));
                         }
                     }
-                    xfce_rc_close(rcfile);
                 }
-                g_free(resource);
+                xfce_rc_close(rcfile);
             }
+            g_free(resource);
         }
     }
 
